# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, print_function

import inflection


class FilterOptionsMixin(object):

    view_options_defaults = {}

    @property
    def view_options_property(self):
        return "{}_options".format(inflection.underscore(self.__class__.__name__))

    def get_options(self, view):
        if not hasattr(self, '_options'):
            self._options = {}
            self._options.update(self.view_options_defaults)
            self._options.update(getattr(view, self.view_options_property, {}))
        return self._options

    def get_option(self, view, option, default=None):
        return self.get_options(view).get(option, default)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, print_function

from rest_framework.serializers import Serializer, ListSerializer


class SourceSerializer(Serializer):

    def to_representation(self, instance):
        return instance['_source']


class RoutedDocTypeMixin(object):

    def get_routing_key(self):
        return self.context.get("routing_key", None)


class DocTypeSerializer(RoutedDocTypeMixin, Serializer):

    def to_representation(self, instance):
        data = super(DocTypeSerializer, self).to_representation(instance)
        data.update(instance.to_dict())
        return data


class DocTypeListSerializer(ListSerializer):

    def to_representation(self, instance):
        return instance.to_dict()


class AggregationsSerializer(Serializer):

    pass


class AggregationSerializer(Serializer):
    """
    Substitutes aggregation bucket to search results and use them in serialization

    The paginator's results will contain the results found in Meta.aggregation_bucket_name

    """

    @classmethod
    def many_init(cls, instance=None, *args, **kwargs):

        if instance is None:
            return super(AggregationSerializer, cls).many_init(instance, *args, **kwargs)

        if not hasattr(instance, 'aggregations'):
            raise RuntimeError("No aggregations found")

        meta = getattr(cls, 'Meta', None)
        try:
            aggregation = instance.aggregations[meta.aggregation_bucket_name].buckets
            return super(AggregationSerializer, cls).many_init(aggregation, **kwargs)
        except AttributeError:
            raise RuntimeError("No aggregation_bucket_name found on serializer's Meta")
        except KeyError:
            raise RuntimeError("Aggregation bucket {} doesn't exist in aggregations".format(meta.aggregation_bucket_name))





# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, print_function
from collections import OrderedDict
from django.core.exceptions import ImproperlyConfigured
from elasticsearch.exceptions import TransportError
from rest_framework.exceptions import ParseError

from rest_framework.utils.urls import replace_query_param
from rest_framework.pagination import _positive_int
from rest_framework.pagination import _get_count, LimitOffsetPagination, BasePagination, CursorPagination
from rest_framework.response import Response


class ElasticsearchPagination(LimitOffsetPagination):

    max_limit = 200
    max_pages = 500

    def paginate_queryset(self, queryset, request, view=None):
        self.limit = self.get_limit(request)
        if self.limit is None:
            return None
        self.offset = self.get_offset(request)

        # try:
        results = queryset[self.offset:self.offset + self.limit].execute()
        # except TransportError:
            # raise ParseError("Invalid query or parameter value")
        self.count = results.hits.total
        if 'suggest' in results:
            self.suggestions = results.suggest.to_dict()
        else:
            self.suggestions = []
        if 'aggregations' in results:
            self.aggregations = results.aggregations.to_dict()
        else:
            self.aggregations = {}
        self.request = request
        if self.count > self.limit and self.template is not None:
            self.display_page_controls = True
        return results

    def get_paginated_response(self, data):
        items = [
            ('count', self.count),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('results', data),
        ]
        if self.suggestions:
            items.append(('suggestions', self.suggestions))
        if self.aggregations:
            items.append(('aggregations', self.aggregations))
        # Allow custom/additional data to be returned in the response if required
        if hasattr(self, 'get_related'):
            items += self.get_related(data).items()
        return Response(OrderedDict(items))

    def get_offset(self, request):
        try:
            return _positive_int(
                request.query_params[self.offset_query_param],
                cutoff=((self.limit * self.max_pages) - self.limit)
            )
        except (KeyError, ValueError):
            return 0

    def get_next_link(self):
        if self.offset + self.limit >= self.count:
            return None

        if self.offset + self.limit >= (self.max_pages * self.limit):
            return None

        url = self.request.build_absolute_uri()
        offset = self.offset + self.limit
        return replace_query_param(url, self.offset_query_param, offset)


class ElasticsearchAggregationPagination(ElasticsearchPagination):
    def get_paginated_response(self, data):
        return Response(OrderedDict([
            ('count', self.count),
            ('next', None),
            ('previous', None),
            ('results', data)
        ]))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, print_function

from collections import OrderedDict
from django.utils import six

from rest_framework.serializers import Serializer
from rest_framework.exceptions import ValidationError
from rest_framework.fields import Field, ChoiceField


class DateHistogramAggregationField(Field):

    def to_bucket_representation(self, bucket):
        return {"date": bucket['key_as_string'], "count": bucket['doc_count']}

    def to_representation(self, value):
        return (self.to_bucket_representation(bucket) for bucket in value['buckets'])


class GeoHashAggregationField(Field):

    def to_representation(self, value):
        return (self.to_bucket_representation(bucket) for bucket in value['buckets'])


class StatsAggregationField(Serializer):

    def to_representation(self, instance):
        return instance.to_dict()


class GeoBoundsAggregationField(Serializer):

    def to_representation(self, value):
        bounds = getattr(value, 'bounds', None)
        if bounds:
            return [bounds['top_left']['lon'], bounds['top_left']['lat']] + [bounds['bottom_right']['lon'], bounds['bottom_right']['lat']]
        return None


class DocTypeField(Field):

    def to_representation(self, value):
        return value.to_dict()


class GeoPointField(Field):

    def to_representation(self, value):
        return tuple(value['geopoint'])

    def to_internal_value(self, data):
        if not isinstance(data, list):
            raise ValidationError('GeoPoint should be a list [lon, lat]')
        if len(data) != 2:
            raise ValidationError('GeoPoint should have 2 values [lon, lat]')
        return {
            'geopoint': data,
            'geoshape': {
                'coordinates': data,
                'type': 'point'
            }
        }


class MappedChoiceField(ChoiceField):

    def __init__(self, *args, **kwargs):
        super(MappedChoiceField, self).__init__(*args, **kwargs)
        self.choice_strings_to_values = dict([
            (six.text_type(key), value) for key, value in self.choices.iteritems()
        ])


class MaxAggregationField(Field):

    def to_representation(self, value):
        return value['value']


class TermsAggregationField(Field):

    def __init__(self, key_mapping={}, *args, **kwargs):
        self.key_mapping = key_mapping
        super(TermsAggregationField, self).__init__(*args, **kwargs)

    def to_bucket_representation(self, bucket_name, bucket):
        return self.key_mapping.get(bucket_name, bucket_name), bucket.doc_count

    def to_representation(self, value):
        return OrderedDict(self.to_bucket_representation(bucket.key, bucket) for bucket in value.buckets)


class RangeAggregationField(Field):

    def to_bucket_representation(self, bucket_name, bucket):
        return bucket_name, bucket['doc_count']

    def to_representation(self, value):
        return OrderedDict(self.to_bucket_representation(bucket_name, bucket) for bucket_name, bucket in value.buckets.to_dict().iteritems())



class TopHitsAggregationField(Field):

    def __init__(self, hit_serializer, *args, **kwargs):
        self.hit_serializer = hit_serializer
        super(TopHitsAggregationField, self).__init__(*args, **kwargs)

    def to_hit_representation(self, hit):
        return self.hit_serializer(hit).data

    def to_representation(self, value):
        return OrderedDict((
            ("count", value.hits.total),
            ("results", (self.to_hit_representation(hit['_source']) for hit in value.hits.hits)),
        ))



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
import os
import codecs
from setuptools import setup, find_packages

def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


VERSION = read("VERSION").strip()

REPO_URL = "https://github.com/predicthq/rfhq"

PYPI_README_NOTE = """\
.. note::

   For the latest source, discussions, bug reports, etc., please visit the `GitHub repository <{}>`_
""".format(REPO_URL)

LONG_DESCRIPTION = "\n\n".join([PYPI_README_NOTE, read("README.rst")])


setup(
    name='rfhq',
    version=VERSION,
    license='MIT',
    author='PredictHQ',
    author_email="developers@predicthq.com",
    long_description=LONG_DESCRIPTION,
    url=REPO_URL,
    packages=find_packages(exclude=('tests*',)),
    test_suite="nose.collector",
    setup_requires=[
        "nose==1.3.7"
    ],
    tests_require=[
        "six",
        "nose==1.3.7",
        "coverage>=4.2",
        "mock==1.3.0",
    ],
    install_requires=[
        "Django>=1.8,<=1.11",
        "djangorestframework>=3",
        "inflection",
        "python-dateutil",
        "elasticsearch<2.0.0",
        "elasticsearch-dsl<2.0.0",
    ],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
    ]
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import, print_function
from rest_framework.generics import GenericAPIView
from rest_framework.response import Response


class AggregateModelMixin(object):
    @property
    def aggregation_backends(self):
        raise NotImplementedError()

    def aggregate_queryset(self, queryset):
        for backend, nested_backends in self.aggregation_backends:
            queryset = backend(children=nested_backends).aggregate_queryset(self.request, queryset, self)
        return queryset

    def aggregate(self, request, *args, **kwargs):
        queryset = self.aggregate_queryset(self.filter_queryset(self.get_queryset()))

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)

        data = serializer.data

        return Response(data)


class AggregateAPIView(AggregateModelMixin, GenericAPIView):
    def get(self, request, *args, **kwargs):
        return self.aggregate(request, *args, **kwargs)


class BaseAggregation(object):

    def __init__(self, children=None):
        self.children = children if children else []

    def get_aggregation_bucket(self, request, queryset, view):
        raise NotImplementedError()

    def get_children(self):
        return self.children

    def aggregate_queryset(self, request, queryset, view):
        bucket = self.get_aggregation_bucket(request, queryset, view)

        for sub_aggregration in self.get_children():
            sub_aggregration().aggregate(request, queryset, view, bucket=bucket)

        return queryset
